/////////////////////////////////////////////////////////////////////////////// 
// Copyright (C) 2003-2011, Open Design Alliance (the "Alliance"). 
// All rights reserved. 
// 
// This software and its documentation and related materials are owned by 
// the Alliance. The software may only be incorporated into application 
// programs owned by members of the Alliance, subject to a signed 
// Membership Agreement and Supplemental Software License Agreement with the
// Alliance. The structure and organization of this software are the valuable  
// trade secrets of the Alliance and its suppliers. The software is also 
// protected by copyright law and international treaty provisions. Application  
// programs incorporating this software must include the following statement 
// with their copyright notices:
//   
//   This application incorporates Teigha(R) software pursuant to a license 
//   agreement with Open Design Alliance.
//   Teigha(R) for .dwg files Copyright (C) 2003-2011 by Open Design Alliance. 
//   All rights reserved.
//
// By use of this software, its documentation or related materials, you 
// acknowledge and accept the above terms.
//
//
// *DWG is the native and proprietary file format for AutoCAD(R) and a trademark  
// of Autodesk, Inc.  Open Design Alliance is not associated with Autodesk.
///////////////////////////////////////////////////////////////////////////////


#ifndef _OD_MODELERGEOMETRY_INCLUDED_
#define _OD_MODELERGEOMETRY_INCLUDED_

#include "RxObject.h"
#include "ModelerDefs.h"
#include "OdDb.h"
#include "DbEntity.h"
#include "Db3dSolid.h"
#include "DbRegion.h"
#include "DbSubDMesh.h"
#include "DbNurbSurface.h"
#include "Ge/GeNurbCurve3d.h"
#include "Ge/GeCurve3dPtrArray.h"
#include "Ge/GeCompositeCurve3d.h"

class OdStreamBuf;
class OdBrBrep;
class OdBrEntity;
class OdGiCommonDraw;
class OdGiViewportDraw;
class OdDbCurve;
class OdDbRevolveOptions;
class OdDbSweepOptions;
class OdDbLoftOptions;
class OdDbSection;
class SectArgs;

#include "TD_PackPush.h"
#include "grx/grxdefs.h"


/** Description:
    This class defines the interface for classes that
    provide conversion services for ACIS SAT and SAB data.

    Remarks:
    This class is used by Teigha to convert ACIS data between various
    versions of SAT and SAB.  
    
    For example, if a version 2004 .dwg file is saved as R15 .dxf file, 
    the SAB data *in* the 2004 file must be converted to version 700 SAT,
    so as to be be valid within the R15 .dxf file.

    Library: ModelerGeometry
    {group:Other_Classes}
*/
class TOOLKIT_EXPORT OdModelerGeometry : public OdRxObject
{
protected:
  OdModelerGeometry() {}

public:  
  ODRX_DECLARE_MEMBERS(OdModelerGeometry);

  // in/out functionality

  /** Description:
      Reads the ACIS data from the specified StreamBuf object into this ModelerGeometry object.

      Remarks:
      The implementation of this function is responsible for determining
      the type (SAT/SAB) and version of the data read.

      Arguments:
      pStreamBuf (I) Pointer to the StreamBuf object from which the data are to be read.   
      typeVer (O) Receives the type and version of the ACIS file.
      standardSaveFlag (I) Controls the saving and restoring of use count data *in* the save file.
      
      Remarks:
      Returns eOk if successful, or an appropriate error code if not.

      pStreamIn may be NULL.
     
      See Also:
      AfTypeVerEnum
  */
  virtual OdResult in(OdStreamBuf* pStreamBuf, AfTypeVer *typeVer = 0, bool standardSaveFlag = true) = 0;
  

  /** Description:
      Writes the ACIS data from this ModelerGeometry object to the specified StreamBuf object.

      Note:
      Implementation of this function are responsible for converting
      the contained ACIS data to the specified type and format.  If afVer == kAfTypeVerAny, 
      then any type or version of may be written.

      Arguments:
      pStreamBuf (I) Pointer to the StreamBuf object to which the data are to be written.   
      typeVer (I) Type and version of the ACIS file.
      standardSaveFlag (I) Controls the saving and restoring of use count data *in* the save file.

      Remarks:
      Returns eOk if successful, or an appropriate error code if not.

      See Also:
      AfTypeVerEnum
  */
  virtual OdResult out(OdStreamBuf* pStreamBuf, AfTypeVer typeVer, bool standardSaveFlag = true) const = 0;

  /** Description:
      Populates the specified BrBrep object with the ACIS BREP data of this ModelerGeometry object.
      Arguments:
      brBrep (O) Receives the BREP data. 
      Remarks:
      Returns true if and only if successful
      
  */
  virtual bool brep(OdBrBrep& brBrep) const = 0;

  virtual OdResult setSubentPath(OdBrEntity & /*pInpEnt*/, OdDbFullSubentPath& /*subpath*/) { return Oda::eNotImplemented; }

  /** Description:
    Draw functionality.
  */
  enum
  {
    kNothing      =  0,
    kIsolines     =  1,
    kEdges        =  2,
    kShells       =  4,
    kOrderedEdges =  8,  //  Geometry is returned as closed loops.
    kSurfIsolines = 16
  };

  /** Description:
    Sets the Facetres for this ModelerGeometry object.
    Arguments:
    facetRes (I) Facet resolution.
  */
  virtual bool setFACETRES(double facetRes) = 0;

  struct NumIsolines
  {
    OdUInt32 numU, numV;
  };
  /** Description:
    Creates a viewport-independent geometric representation of this ModelerGeometry object.
    
    Remarks:
    Returns true if and only if the geometric representation can be generated *in* a
    viewport-independent manner. 

    Note:
    A return value of false indicates that viewportDraw() must be called of this object.

    The 3D GS will call this function at least once, but may cache subsequent *display* updates.

    Arguments:
    pWd (I) Pointer to the OdGiCommonDraw interface.
    geomType (I) Geometry type.
    pNumIsolines (I) number of isolines in U and V directions.
    
    Remarks:
    geomType must be a combination of one or more of the following:
    
    @table
    Name            Value
    kNothing        0   
    kIsolines       1
    kEdges          2
    kShells         4
    kOrderedEdges   8
    kSurfIsolines   16
  */
  virtual bool worldDraw(OdGiCommonDraw* pWd, OdUInt32 geomType, const NumIsolines *pNumIsolines = 0) = 0;
  
  /** Description:
    Creates a viewport-dependent silhouette representation of this ModelerGeometry object.

    Remarks:
    Causes this ModelerGeometry object to describe its geometry to the specified OdGiViewportDraw object. 

    This function is called once per viewport.

    Remarks:
    it is possible for have totally different images *in* each viewport; a schematic
    *in* one viewport, a layout *in* another.
    
    Arguments:
    pVd (I) Pointer to the OdGiViewportDraw interface.
  */
  virtual bool drawSilhouettes(OdGiViewportDraw* pVd) = 0;

  /** Description:
    Explodes this ModelerGeometry object into a set of OdDbEntity objects.  

    Arguments:
    entitySet (I/O) Receives an array of pointers to the new entities.
    
    Remarks:
    Returns eOk if successful, or an appropriate error code if not.

    Note:
    Entities resulting from the explosion are appended to the specified array.
    
    The newly created entities are not *database* resident.
  */
  virtual bool explode(OdDbEntityPtrArray& entitySet) const = 0;
  
  /** Description:
    Returns the transformation matrix for this ModelerGeometry object.
    Arguments:
    xfm (O) Receives the transformation matrix.
  */
  virtual bool getTransformation(OdGeMatrix3d& xfm) = 0;
  
  /**
    Description:
    Applies the specified 3D transformation matrix to this ModelerGeometry object.

    Arguments:
    xfm (I) 3D transformation matrix.
  */
  virtual void transformBy( const OdGeMatrix3d& xfm ) = 0;
  
  /** Description:
    Creates a WCS aligned solid box centered about the world origin. 
    
    Arguments:
    xLen (I) Dimension along the WCS X-axis.
    yLen (I) Dimension along the WCS Y-axis. 
    zLen (I) Dimension along the WCS Z-axis. 

    Remarks:
    Returns eOk if successful, or an appropriate error code if not. 
    
    Note:
    The following constraints apply:
    
    o xLen >= 1e-6.
    o yLen >= 1e-6.
    o zLen >= 1e-6.
  */
  virtual void createBox( double xLen, double yLen, double zLen ) = 0;
  /** Description
    Creates a WCS aligned frustum centered about the world origin. 
    
    Arguments:
    height (I) Dimension along the WCS Z-axis.
    xRadius (I) Base radius along the WCS X-axis. 
    yRadius (I) Base radius along the WCS Y-axis.
    topXRadius (I) Top radius along the WCS X-axis. 

    Remarks:
    The function can generate a frustum that is any one of the following:
    
    o A circular cylinder
    o An elliptical cylinder
    o A circular cone
    o An elliptical cone
    o A truncated circular cone
    o A truncated elliptical cone
    
    Note:    
    The following constraints apply:
    
    o height >= 1e-6.
    o xRadius >= 1e-6.
    o yRadius >= 1e-6.
    o topXRadius >= 1e-6. 
  */
  virtual void createFrustum( double height, double xRadius, double yRadius, double topXRadius ) = 0;
  /** Description:
    Creates a sphere with centered about the world origin. 
    
    Arguments:
    radius (I) Radius.

    Note:
    The following constraints apply:
    
    o radius >= 1e-6.

  */
  virtual void createSphere( double radius ) = 0;
  /** Description:
    Creates a WCS aligned torus with centered about the world origin
    and the Z-axis as its axis of rotation. 
    
    Arguments:
    majorRadius (I) Radius of the torus
    minorRadius (I) Radius of the tube.

    Note:
    The following constraints apply:
    
    o minorRadius >= 1e-6.
    o |majorRadius| >= 1e-6
    o if majorRadius < 0, then minorRadius > |majorRadius| + 1e-6
  */
  virtual void createTorus( double majorRadius, double minorRadius ) = 0;

  /** Description:
    Creates a WCS aligned wedge centered about the world origin. 
    
    Arguments:
    xLen (I) Dimension along the WCS X-axis.
    yLen (I) Dimension along the WCS Y-axis. 
    zLen (I) Dimension along the WCS Z-axis. 

    Remarks:
    The inclined face is defined by the points 
    
              ( xLen/2, -yLen/2, -zLen/2)
              (-xLen/2, -yLen/2,  zLen/2)  
              ( xLen/2,  yLen/2, -zLen/2)
              (-xLen/2, +yLen/2,  zLen/2)  
    
    Note:
    The following constraints apply:
    
    o xLen >= 1e-6.
    o yLen >= 1e-6.
    o zLen >= 1e-6.
  */
  virtual void createWedge( double xLen, double yLen, double zLen ) = 0;

  /** Description:
    Creates a solid by extruding the specified region.
   
    Arguments:
    pRegion (I) Pointer to the region to be extruded.
    height (I) Height of the extrusion.
    taperAngle (I) Taper angle.

    Remarks:
    The extrusion direction is along the normal of the region. height may be negative.     


    Note:
    All angles are expressed *in* radians.
     
    The following constraints apply:
    
    o |height| >= 1e-6.
    o |taperAngle| < OdaPI2 - 1e-6.
  */
  virtual OdResult extrude(const OdDbRegion* pRegion, double height, double taperAngle, bool isSolid = true) = 0;

  /** Description:
    Creates a solid by revolving the specified region.
   
    Arguments:
    pRegion (I) Pointer to the region to be extruded.
    axisPoint (I) Point on the axis of revolution.
    axisDir (I) Vector defining the axis of revolution.
    angleOfRevolution (I) Angle of revolution.
    
    Remarks:
    The axis of revolution is projected onto the plane of the region, parallel to its normal.

    Note:
    All angles are expressed *in* radians.
    
    The following constraints apply:
    
    o The projected axis of revolution cannot intersect the region. 
    o angleOfRevolution >= 1e-6
  */
  virtual OdResult revolve(const OdDbRegion* pRegion, const OdGePoint3d& axisPoint, 
    const OdGeVector3d& axisDir, double angleOfRevolution, bool isSolid = true) = 0;

  /** Description:
    Clears the color attributes of this ModelerGeometry object.
  */
  virtual void ClearColorAttributes() = 0;

  /** Description:
  Clears the material attributes of this ModelerGeometry object.
  */
  virtual void ClearMaterialAttributes() = 0;

  /** Description:
  Clears the material mapper attributes of this ModelerGeometry object.
  */
  virtual void ClearMaterialMapperAttributes() = 0;

  /** Description
    Returns true if and only if TrueColor attributes are attached to the faces and/or
    edges of this ModelerGeometry object.
  */
  virtual bool hasTrueColorAttributes() const = 0;

  /** Description:
    Returns the *plane* of this ModelerGeometry object.

    Arguments:
    regionPlane (O) Receives the region *plane*.
    Remarks:
    Returns eOk if successful, or an appropriate error code if (bodyType() != kRegion).
    
  */
  virtual OdResult getPlane(OdGePlane& regionPlane) const = 0;

  /** Description:
    Sets the mm/unit for this ModelerGeometry object.
    Arguments:
    mmPerUnit (I) mm/unit. 
    Remarks:
    Returns true if and only if successful.
  */
  virtual bool setMMPerUnit(double  mmPerUnit) = 0;
  /** Description:
    Returns the mm/unit for this ModelerGeometry object.
    Arguments:
    mmPerUnit (O) mm/unit.
    Remarks:
    Returns true if and only if successful.
  */
  virtual bool getMMPerUnit(double &mmPerUnit) const = 0;

  enum geomType
  {
    kUndefined = 0,
    kBody      = 1,
    kSolid     = 2,
    kRegion    = 3,
    kSurface   = 4
  };

  /** Description:
    Returns the body type of this ModelerGeometry object:
    Remarks:
    bodyType returns one of the following:
    
    @table
    Name         Value
    kUndefined   0
    kBody        1
    kSolid       2
    kRegion      3
  */
  virtual geomType bodyType() const = 0;

  enum solidGeomType
  {
    kSphere  = 0,
    kTorus   = 1,
    kPyramid = 2,
    kBox     = 3,
    kWedge   = 4,
    kOther   = 5
  };

  virtual solidGeomType solidType() const = 0;

  // new methods

  virtual OdResult booleanOper(OdDb::BoolOperType operation, OdSmartPtr<OdModelerGeometry> otherRegion) = 0;
  virtual OdResult booleanOper(OdDb::BoolOperType operation, const OdSmartPtr<OdModelerGeometry> pModelerPar, OdDbEntityPtrArray& intersectionEntities) = 0;
  
  virtual OdResult getArea(double& regionArea) const = 0;

  virtual OdResult getPerimeter(double&) const = 0;

  virtual OdResult getAreaProp( const OdGePoint3d& origin, const OdGeVector3d& xAxis,
    const OdGeVector3d& yAxis, double& perimeter,
    double& area, OdGePoint2d& centroid,
    double momInertia[2], double& prodInertia,
    double prinMoments[2], OdGeVector2d prinAxes[2],
    double radiiGyration[2], OdGePoint2d& extentsLow,
    OdGePoint2d& extentsHigh ) const = 0;
  
  virtual OdResult clear( ) = 0;

  virtual OdResult extrudeAlongPath(const OdDbRegion* region, const OdDbCurve* path, double taperAngle = 0.0, bool isSolid = true) = 0;

  virtual OdResult imprintEntity(const OdDbEntity *pEntity) = 0;

  virtual OdResult checkInterference(const OdDb3dSolid* otherSolid,
    OdBool createNewSolid, 
    OdBool& solidsInterfere,
    OdDb3dSolidPtr &commonVolumeSolid)
    const = 0;

  virtual OdResult getMassProp(double& volume,
    OdGePoint3d& centroid,
    double momInertia[3],
    double prodInertia[3],
    double prinMoments[3],
    OdGeVector3d prinAxes[3], 
    double radiiGyration[3],
    OdGeExtents3d& extents) const = 0;

  
  /** \details
    Creates a DbRegion entity as a result of the intersection between the specified plane and the ModelerGeometry object. 

    \param plane          [in] Reference to an OdGePlane object to use as the section cutting plane.
    \param sectionRegion  [out] Reference to a new region created as the result of the intersection between the specified plane and the ModelerGeometry object.
    \param pSectionCurves [out] Pointer to an array of section curves.
 
    \returns Returns the result of intersecting the ModelerGeometry object with a 
    plane: eOk if successful or an appropriate error code in other case. 
    Value eOk is also returned when a plane and the ModelerGeometry object do not 
    intersect each other or if the intersection region is not a valid region, that is, 
    it intersects at a point, along a bounding face of the ModelerGeometry object and so on. 
  */
  virtual OdResult getSection(const OdGePlane& plane, OdDbRegionPtr &sectionRegion, OdDbEntityPtrArray* pSectionCurves = NULL) const = 0;
  //virtual OdResult getSection(const OdGePlane& plane, OdDbRegion* &sectionRegion) const = 0;
  virtual OdResult getSectionByProject(const OdGePlane& plane, OdDbEntityPtrArray& sectionPrjCurves) const = 0;

  virtual OdResult getSlice(const OdGePlane& plane, bool getNegHalfToo, OdDb3dSolid* &negHalfSolid) = 0;
  virtual OdResult getSlice(OdDbSurface* pSurface, bool bGetNegHalfToo, OdDb3dSolid*& pNegHalfSolid) = 0;

  virtual OdResult getPlaneSliceFaceId(int &faceId, OdGeIntArray &edgeIt) = 0;

  virtual OdResult cleanBody() = 0;
  virtual OdResult offsetBody(double offsetDistance) = 0;

  virtual OdResult separateBody(OdArray<OdDb3dSolid *> &newSolids) = 0;

  virtual OdResult createExtrudedObject ( OdDbEntity* pSweepEnt,
                                          const OdGeVector3d& directionVec,
                                          OdDbSweepOptions& sweepOptions,
                                          bool isSolid = true) = 0;

  virtual OdResult createLoftedObject (
    OdDbEntityPtrArray& crossSectionCurves,
    OdDbEntityPtrArray& guideCurves,
    OdDbEntity* pPathCurve, 
    OdDbLoftOptions& loftOptions,
    bool isSolid = true) = 0;

  virtual OdResult createPyramid(double height, int sides, 
    double radius, double topRadius = 0.0 ) = 0;

  virtual OdResult createRevolvedObject ( OdDbEntity* pRevEnt,
    const OdGePoint3d& axisPnt,
    const OdGeVector3d& axisDir,
    double revAngle, double startAngle,
    OdDbRevolveOptions& revolveOptions,
    bool isSolid = true) = 0;

  virtual OdResult createSweptObject ( OdDbEntity* pSweepEnt,
    OdDbEntity* pPathEnt,
    OdDbSweepOptions& sweepOptions,
    bool isSolid = true) = 0;

  // Loft options utility functions
  virtual OdResult checkCrossSectionCurves (
    OdDbEntityPtrArray& crossSectionCurves,
    bool& allOpen, bool& allClosed, bool& allPlanar,
    bool displayErrorMessages = false ) = 0;
  
  virtual OdResult checkGuideCurves ( OdDbEntityPtrArray& guideCurves,
    bool displayErrorMessages = false ) = 0;
  
  virtual OdResult checkPathCurve ( OdDbEntity *pPathCurve,
    bool displayErrorMessages = false ) = 0;

  // Sweep options utility functions
  virtual OdResult  checkSweepCurve ( OdDbEntity *pSweepEnt,
    OdDb::Planarity& planarity, OdGePoint3d& pnt, OdGeVector3d& vec,
    bool& closed, double& approxArcLen, bool displayErrorMessages = false ) = 0;

  virtual OdResult  checkSweepPathCurve ( OdDbEntity *pPathEnt,    bool displayErrorMessages = false ) = 0;

  // Revolve options utility functions
  virtual OdResult checkRevolveCurve(
    OdDbEntity* pRevEnt,
    const OdGePoint3d& axisPnt,
    const OdGeVector3d& axisDir,
    bool& closed,
    bool& endPointsOnAxis,
    bool& planar,
    bool displayErrorMessages = false) = 0;

  /** Description: 
  Creates an extruded surface when given a planar entity with thickness, 
  or creates an plane surface when given a a simple, closed planar entity.

  Arguments:
  pFromEntity (I) pointer to either a planar entity with thickness, a simple, closed planar entity, or an OdDbSolid entity 
  pointed to by pFromEntity, or null if the operation fails 

  Remarks:

  Returns eOk if successful, or an appropriate error code if not. 

  Note:
  */
  virtual OdResult createFrom ( const OdDbEntity* pFromEntity ) = 0;

  virtual OdResult convertToRegion ( OdDbEntityPtrArray& regions ) = 0;
  virtual OdResult thicken(double thickness, bool bBothSides, OdDb3dSolid*& pSolid) const = 0;

  virtual OdResult intersectWith( const OdDbEntity* ent, OdDb::Intersect intType, 
    OdGePoint3dArray& points, OdGsMarker thisGsMarker, OdGsMarker otherGsMarker ) const = 0;

  virtual OdResult intersectWith( const OdDbEntity* ent, OdDb::Intersect intType, const OdGePlane& projPlane, 
    OdGePoint3dArray& points, OdGsMarker thisGsMarker, OdGsMarker otherGsMarker ) const = 0;

  virtual OdResult copySubEntity(const OdDbSubentId &subentId, OdDbEntityPtr &newEntity) const = 0;
  virtual OdResult taperFaces(const OdArray<OdDbSubentId *> &faceSubentIds, const OdGePoint3d &basePoint, 
                              const OdGeVector3d &draftVector, double draftAngle) = 0;
  virtual OdResult removeFaces(const OdArray<OdDbSubentId *> &faceSubentIds) = 0;
  virtual OdResult offsetFaces(const OdArray<OdDbSubentId *> &faceSubentIds, double offsetDistance) = 0;
  virtual OdResult shellBody(const OdArray<OdDbSubentId *> &faceSubentIds, double offsetDistance) = 0;
  virtual OdResult transformFaces(const OdArray<OdDbSubentId *> &faceSubentIds, const OdGeMatrix3d &matrix) = 0;

  virtual OdResult getSubentPathsAtGsMarker(OdDb::SubentType type,
                                            OdGsMarker gsMark, 
                                            const OdGePoint3d& pickPoint,
                                            const OdGeMatrix3d& viewXform, 
                                            OdDbFullSubentPathArray& subentPaths, 
                                            const OdDbObjectIdArray* entAndInsertStack = 0) const = 0;
  virtual OdResult getGsMarkersAtSubentPath(const OdDbFullSubentPath& subPath, 
                                            OdArray<OdGsMarker>& gsMarkers) const = 0;

  virtual OdDbSubentId internalSubentId(void* ent) const = 0;
  virtual void* internalSubentPtr(const OdDbSubentId& id) const = 0;

  virtual bool getNurbCurvesCache(OdGeCurve3dPtrArray &)  const { return false; }
  virtual bool updateisoline() const {return false;}
  virtual void setDType(int type){};

  // for internal use only
  virtual OdResult setBody(const void * /*pBody*/) { return Oda::eNotImplemented; }
  virtual void * body( ) const { return 0; }
  virtual OdResult getGripPoints( OdGePoint3dArray& gripPoints ) const { return Oda::eInvalidInput; }

  virtual OdResult getOsnapPoints(OdDb::OsnapMode osnapMode, 
    OdDb::SubentType markerType,
    int gsSelectionMark, 
    const GcGePoint3d& pickPoint, 
    const GcGePoint3d& lastPoint, 
    const GcGeMatrix3d& xWorldToEye, 
    GcGePoint3dArray& snapPoints
    ) const { return Oda::eInvalidInput; }

  virtual OdResult setSubentColor(const OdDbSubentId &subentId, const OdCmColor &color) = 0;
  virtual OdResult getSubentColor(const OdDbSubentId &subentId, OdCmColor &color) const = 0;
  virtual OdResult setSubentMaterial(const OdDbSubentId &subentId, OdDbObjectId matId) = 0;
  virtual OdResult getSubentMaterial(const OdDbSubentId &subentId, OdUInt64& matId) const = 0;
  virtual OdResult setSubentMaterialMapper( const OdDbSubentId &subentId, 
                                            OdGeMatrix3d &mx, 
                                            OdUInt8 &projection, 
                                            OdUInt8 &tiling, 
                                            OdUInt8 &autoTransform) = 0;
  virtual OdResult getSubentMaterialMapper( const OdDbSubentId &subentId,
                                            OdGeMatrix3d &mx, 
                                            OdUInt8 &projection, 
                                            OdUInt8 &tiling, 
                                            OdUInt8 &autoTransform) const = 0;

  virtual OdResult chamferEdges(const OdArray<OdDbSubentId *> &edgeSubentIds,
                                const OdDbSubentId& baseFaceSubentId, 
                                double baseDist, 
                                double otherDist) = 0;

  virtual OdResult filletEdges( const OdArray<OdDbSubentId *> &edgeSubentIds,
                                const OdGeDoubleArray& radius, 
                                const OdGeDoubleArray& startSetback, 
                                const OdGeDoubleArray& endSetback) = 0;

  virtual OdResult createSectionObjects(const OdGePlane& sectionPlane, OdDbEntityPtrArray& sectionObjects) const = 0;

  virtual OdResult createInterferenceObjects( OdDbEntityPtrArray& interferenceObjects,
                                              OdDbEntityPtr pEntity, 
                                              unsigned int flags ) const = 0;

  virtual OdResult sliceByPlane(const OdGePlane& slicePlane, OdDbSurfacePtr& pNegHalfSurface, OdDbSurfacePtr& pNewSurface, bool bNotModifyItself) = 0;
  virtual OdResult sliceBySurface(const OdDbSurfacePtr pSlicingSurface, OdDbSurfacePtr& pNegHalfSurface, OdDbSurfacePtr& pNewSurface, bool bNotModifyItself) = 0;

  virtual OdResult extrudeFaces(const OdArray<OdDbSubentId *> &faceSubentIds, double height, double taper) = 0;
  virtual OdResult extrudeFacesAlongPath(const OdArray<OdDbSubentId *> &faceSubentIds, const OdDbCurve* path) = 0;
  
  virtual OdResult ChangeFacesDoubleSidedParam(bool isDoubleSided) = 0;

//  virtual OdResult convertToNurbSurface(OdDbNurbSurfaceArray& nurbSurfaceArray) = 0;

  virtual OdResult get( int& iUDegree, int& iVDegree, bool& bRational, int& iUNumControlPoints, int& iVNumControlPoints,
                        OdGePoint3dArray& ctrlPtsArr, OdGeDoubleArray& weights, 
                        OdGeKnotVector& uKnots, OdGeKnotVector& vKnots) const = 0;

  virtual OdResult set (int iUDegree, int iVDegree, bool bRational, int iUNumControlPoints, int iVNumControlPoints,
                        const OdGePoint3dArray& ctrlPtsArr, const OdGeDoubleArray& weights, 
                        const OdGeKnotVector& uKnots, const OdGeKnotVector& vKnots) = 0;
  
  virtual OdResult getNumberOfControlPointsInU(int& iCount) const = 0;
  virtual OdResult getNumberOfControlPointsInV(int& iCount) const = 0;

  virtual OdResult getNumberOfKnotsInU(int& iCount) const = 0;
  virtual OdResult getNumberOfKnotsInV(int& iCount) const = 0;

  virtual OdResult getUKnots(OdGeKnotVector& knots) const = 0;
  virtual OdResult getVKnots(OdGeKnotVector& knots) const = 0;

  virtual OdResult getDegreeInU(int& iDegree) const = 0;
  virtual OdResult getDegreeInV(int& iDegree) const = 0;

  virtual OdResult isClosedInU(bool& bIsClosed) const = 0;
  virtual OdResult isClosedInV(bool& bIsClosed) const = 0;

  virtual OdResult isPeriodicInU(bool& bIsPeriodic) const = 0;
  virtual OdResult isPeriodicInV(bool& bIsPeriodic) const = 0;

  virtual OdResult getPeriodInU(double& dPeriod) const = 0; 
  virtual OdResult getPeriodInV(double& dPeriod) const = 0;

  virtual OdResult evaluate(double dU, double dV, OdGePoint3d& pos) const = 0;
  virtual OdResult evaluate(double dU, double dV, OdGePoint3d& pos, OdGeVector3d& uDeriv, OdGeVector3d& vDeriv) const = 0;
  virtual OdResult evaluate(double dU, double dV, OdGePoint3d& pos, OdGeVector3d& uDeriv, OdGeVector3d& vDeriv,
                            OdGeVector3d& uuDeriv, OdGeVector3d& uvDeriv, OdGeVector3d& vvDeriv) const = 0;
  virtual OdResult evaluate(double dU, double dV, int iDerivDegree, OdGePoint3d& point, OdGeVector3dArray& derivatives) const = 0;

  virtual OdResult isRational(bool& bIsRational) const = 0;

  virtual OdResult isPlanar(bool& bIsPlanar, OdGePoint3d& ptOnSurface, OdGeVector3d& normal) const = 0;

  virtual OdResult isPointOnSurface(const OdGePoint3d& point, bool& bOnSurface) const = 0;

  virtual OdResult getNormal(double dU, double dV, OdGeVector3d& normal) const = 0;

  virtual OdResult getNumberOfSpansInU(int& iSpan) const = 0;
  virtual OdResult getNumberOfSpansInV(int& iSpan) const = 0;

  virtual OdResult getIsolineAtU(double dNumberSegments, OdDbCurvePtrArray& lineSegments) const = 0;
  virtual OdResult getIsolineAtV(double dNumberSegments, OdDbCurvePtrArray& lineSegments) const = 0;

  virtual OdResult InsertKnot(double dVal, int iUorV) = 0;

  virtual OdResult InsertControlPointsAtU(double dU, const OdGePoint3dArray& vCtrlPts, const OdGeDoubleArray& vWeights) = 0;
  virtual OdResult InsertControlPointsAtV(double dV, const OdGePoint3dArray& uCtrlPts, const OdGeDoubleArray& uWeights) = 0;

  virtual OdResult RemoveControlPointsAtU(int iUDegree) = 0;
  virtual OdResult RemoveControlPointsAtV(int iVDegree) = 0;

  virtual OdResult rebuild(int iUDegree, int iVDegree, int iNumUCtrlPts, int iNumVCtrlPts, bool bRestore) = 0;

  virtual OdResult modifyPositionAndTangent(double dU, double dV, const OdGePoint3d& point, const OdGeVector3d* uDeriv, const OdGeVector3d* vDeriv) = 0;

  virtual OdResult getParameterOfPoint(const OdGePoint3d& point, double& dU, double& dV) const = 0;

  virtual OdResult getControlPoints(int& iUCount, int& iVCount, OdGePoint3dArray& points) const = 0;

  virtual OdResult setControlPoints(int iUCount, int iVCount, const OdGePoint3dArray& points) = 0;

  virtual OdResult getControlPointAndWeight(int iUIndex, int iVIndex, OdGePoint3d& point, double& weight, bool& bIsRational) const = 0;

  virtual OdResult setControlPointAndWeight(int iUIndex, int iVIndex, const OdGePoint3d& point, double weight) = 0;

  virtual OdResult setColorToSubents(OdCmColor const& color) = 0;

  virtual OdResult setMaterialToSubents(OdDbObjectId materialId) = 0;

  virtual OdResult setMaterialMapperToSubents(OdGeMatrix3d &mx, OdUInt8 &projection,
                                              OdUInt8 &tiling, OdUInt8 &autoTransform) = 0;

  virtual OdResult selectFaces( int selType, const OdGePoint3dArray &selPnts, const OdGeVector3d &viewDir, 
    OdArray<OdDbSubentId*>& faceSet, OdUInt32 flags = 0, const OdGeTol &tol = OdGeTol(1.0e-6) ) = 0;

  virtual OdResult generateSectionGeometry(const OdDbSection*      pSection,
                                           OdDbEntityPtrArray&     sourceEntArr,
                                           OdArray<OdDbEntityPtr>& intBoundaryEnts,
                                           OdArray<OdDbEntityPtr>& intFillEnts,
                                           OdArray<OdDbEntityPtr>& backgroundEnts,
                                           OdArray<OdDbEntityPtr>& foregroundEnts,
                                           OdArray<OdDbEntityPtr>& curveTangencyEnts,
                                           bool                    bIsSetProperties) = 0;
  /** \details 
    Creates a ray which starts from a specified point and has a specified direction.

    \param rayBasePoint [in]  A base point to start a ray from.
    \param rayDir      [in]  A ray direction.
    \param rayRadius   [in]  A ray radius. It defines the tolerance used for intersection checking.
    \param subEntIds   [out] A reference to subentities array to return. Returned subentities can be one of the following: faces, edges, or vertices.
    \param parameters   [out] An array of parameters to return. Parameters shows the intersection between a ray and subentities.

    \returns Returns eOk if a ray was successfully created or an appropriate error code in the other case.
  */
  virtual OdResult rayTest( const OdGePoint3d&      rayBasePoint, 
                            const OdGeVector3d&     rayDir,
                            double                  rayRadius, 
                            OdArray<OdDbSubentId> & subEntIds, 
                            OdGeDoubleArray&        parameters) const = 0;

  /** \details 
    Returns the bounding box.

    \param box [out] 3D bounding boxes as minimum and maximum 3D points.

    \returns Returns eOk if successful or an appropriate error code in the other case.
  */                                  
  virtual OdResult getBoundingBox(OdGeExtents3d &box) = 0;

  /** \details 
    For internal use only.
  */ 
  virtual OdResult generateSectionGeometry( SectArgs& sectArgs, OdDbEntity *pEnt, bool* bHasForeground ) = 0;

};

/** Description:
    This template class is a specialization of the OdSmartPtr class for OdModelerGeometry object pointers.
*/
typedef OdSmartPtr<OdModelerGeometry> OdModelerGeometryPtr;

#include "TD_PackPop.h"

#endif // _OD_MODELERGEOMETRY_INCLUDED_
